@@ -73,6 +73,7 @@ const _urlRegexp = new RegExp(
         ')' +
     ')', 'gi');
 
+const _channelRegexp = new RegExp('(^| )#([\\w\\+\\.-]+)','g');
 
 let debugInit = false;
 let debugEnabled = false;
@@ -108,13 +109,6 @@ function getURISchemes() {
     return schemes;
 }
 
-function addJSSignalMethods(proto) {
-    proto.connectJS = Signals._connect;
-    proto.disconnectJS = Signals._disconnect;
-    proto.emitJS = Signals._emit;
-    proto.disconnectAllJS = Signals._disconnectAll;
-}
-
 function getTpEventTime() {
     let time = Gtk.get_current_event_time ();
     if (time == 0)
@@ -169,6 +163,15 @@ function findUrls(str) {
     return res;
 }
 
+function findChannels(str, server) {
+    let res = [], match;
+    while ((match = _channelRegexp.exec(str)))
+        res.push({ url: 'irc://%s/%s'.format(server, match[2]),
+                   name: '#' + match[2],
+                   pos: match.index + match[1].length });
+    return res;
+}
+
 function openURL(url, timestamp) {
     let ctx = Gdk.Display.get_default().get_app_launch_context();
     ctx.set_timestamp(timestamp);
